<?php

class LinkerConverter extends Converter
{
    private static $handle = null;

    function getInstance()
    {
        // If handle does not exist then create it
        if (!self::$handle)
        {
            self::$handle = new LinkerConverter;
        }

        // Return handle
        return self::$handle;
    }

    function convertAll()
    {
        // Initialize variable used to store results
        $results = array();

        try
        {
            // Initialize converter
            $this->init();

            // Initialize variable
            $results['users'] = array();

            // Convert users
            $this->convertUsers();

            // Initialize variable
            $results['categories'] = array();

            // Convert categories
            $this->convertCategories();

            // Initialize variable
            $results['sites'] = array();

            // Convert sites
            $this->convertSites();

            // Build parents for categories
            $this->buildCategoryParents();
        }
        catch (Exception $e)
        {
            // Store exception in array
            $results[end(array_keys($results))]['error'] = $e->getMessage();
        }

        // Return results
        return $results;
    }

    function convertCategories()
    {
        // Set-up old database
        $this->setOldDb();

        // Get list of linker categories
        $result = $this->query('SELECT * FROM `' . $this->dbOldPrefix . 'CAT` WHERE `sid` > 0', $this->dbOld);

        // Set-up new database
        $this->setNewDb();

        // Delete all categories from arfooo then modify 'category id' and 'parent category id' fields to mediumint(9)
        $this->query('DELETE FROM `' . $this->dbNewPrefix . 'categories`', $this->dbNew);
        $this->query('ALTER TABLE `' . $this->dbNewPrefix . 'categories` CHANGE `categoryId` `categoryId` MEDIUMINT(9) NOT NULL AUTO_INCREMENT', $this->dbNew);
        $this->query('ALTER TABLE `' . $this->dbNewPrefix . 'categories` CHANGE `parentCategoryId` `parentCategoryId` MEDIUMINT(9) NOT NULL', $this->dbNew);

        // Set table name and table prefix
        $this->qObiect->tableName = 'categories';
        $this->qObiect->dbPrefix = $this->dbNewPrefix;

        // Loop through linker categories to store them in arfooo
        while ($row = mysql_fetch_assoc($result))
        {
            // Reset insert query's object
            $this->qObiect->reset();

            // Set values for needed fields
            $this->qObiect->add('name', $row['title']);
            $this->qObiect->add('categoryId', $row['sid']);
            $this->qObiect->add('description', $row['descript']);
            $this->qObiect->add('parentCategoryId', $row['parent']);
            $this->qObiect->add('imageSrc', 'defaultCategoryImage.gif');
            $this->qObiect->add('possibleTender', $row['c_link'], true);

            // Build SQL query
            $sql = $this->qObiect->getSQL();
            $sql = iconv('ISO-8859-1', 'utf-8//ignore', $sql);
            $sql = str_replace("\xc2\x80", "\xe2\x82\xac", $sql);

            // Run query
            $this->query($sql, $this->dbNew);
        }

        // By default return true
        return true;
    }

    function convertSites()
    {
        // Set-up old database
        $this->setOldDb();

        // Get list of linker's validated/waiting sites
        $result = $this->query('SELECT * FROM `' . $this->dbOldPrefix . 'LINK`', $this->dbOld);
        $result2 = $this->query('SELECT * FROM `' . $this->dbOldPrefix . 'WAIT`', $this->dbOld);

        // Set-up new database
        $this->setNewDb();

        // Delete all sites/votes/comments from arfooo then modify 'site id' and 'category id' fields to int(11) for sites table
        $this->query('DELETE FROM `' . $this->dbNewPrefix . 'sites`', $this->dbNew);
        $this->query('DELETE FROM `' . $this->dbNewPrefix . 'votes`', $this->dbNew);
        $this->query('DELETE FROM `' . $this->dbNewPrefix . 'comments`', $this->dbNew);
        $this->query('ALTER TABLE `' . $this->dbNewPrefix . 'sites` CHANGE `siteId` `siteId` INT(11) NOT NULL AUTO_INCREMENT', $this->dbNew);
        $this->query('ALTER TABLE `' . $this->dbNewPrefix . 'sites` CHANGE `categoryId` `categoryId` INT(11) NOT NULL', $this->dbNew);

        // Set table name and table prefix
        $this->qObiect->tableName = 'sites';
        $this->qObiect->dbPrefix = $this->dbNewPrefix;

        // Loop through linker's validated sites to store them in arfooo
        while ($row = mysql_fetch_assoc($result))
        {
            // Webmaster's email
            $email = $row['email'];

            // Reset insert query's object
            $this->qObiect->reset();

            // Set values for needed fields
            $this->qObiect->add('url', $row['url']);
            $this->qObiect->add('siteId', $row['sid']);
            $this->qObiect->add('status', 'validated');
            $this->qObiect->add('webmasterEmail', $email);
            $this->qObiect->add('siteTitle', $row['title']);
            $this->qObiect->add('categoryId', $row['parent']);
            $this->qObiect->add('description', $row['lcont']);
            $this->qObiect->add('visitsCount', $row['visit']);
            $this->qObiect->add('webmasterName', $row['name']);
            $this->qObiect->add('creationDate', date('Y-m-d H:i:s', $row['uptime']));

            // If webmaster's email exists then get his id to set it in data
            if ($email)
            {
                $result3 = $this->query('SELECT `userId` FROM `' . $this->dbNewPrefix . 'users` WHERE `email` = "' . $email . '" AND `role` = "webmaster"', $this->dbNew);
                $row2 = mysql_fetch_assoc($result3);

                if (is_array($row2) && isset($row2['userId']))
                {
                    $this->qObiect->add('webmasterId', $row2['userId']);
                }
            }

            // Build SQL query
            $sql = $this->qObiect->getSQL();
            $sql = iconv('ISO-8859-1', 'utf-8//ignore', $sql);
            $sql = str_replace("\xc2\x80", "\xe2\x82\xac", $sql);

            // Run query
            $this->query($sql, $this->dbNew);
        }

        // Loop through linker's waiting sites to store them in arfooo
        while ($row = mysql_fetch_assoc($result2))
        {
            // Webmaster's email
            $email = $row['email'];

            // Reset insert query's object
            $this->qObiect->reset();

            // Set values for needed fields
            $this->qObiect->add('url', $row['url']);
            $this->qObiect->add('status', 'waiting');
            $this->qObiect->add('webmasterEmail', $email);
            $this->qObiect->add('siteTitle', $row['title']);
            $this->qObiect->add('categoryId', $row['parent']);
            $this->qObiect->add('description', $row['lcont']);
            $this->qObiect->add('webmasterName', $row['name']);

            // If webmaster's email exists then get his id to set it in data
            if ($email)
            {
                $result3 = $this->query('SELECT `userId` FROM `' . $this->dbNewPrefix . 'users` WHERE `email` = "' . $email . '" AND `role` = "webmaster"', $this->dbNew);
                $row2 = mysql_fetch_assoc($result3);

                if (is_array($row2) && isset($row2['userId']))
                {
                    $this->qObiect->add('webmasterId', $row2['userId']);
                }
            }

            // Build SQL query
            $sql = $this->qObiect->getSQL();
            $sql = iconv('ISO-8859-1', 'utf-8//ignore', $sql);
            $sql = str_replace("\xc2\x80", "\xe2\x82\xac", $sql);

            // Run query
            $this->query($sql, $this->dbNew);
        }

        // By default return true
        return true;
    }

    function convertUsers()
    {
        // Initialize variables used to store list of new (arfooo) users
        $users = array();

        // Set-up old database
        $this->setOldDb();

        // Get list of linker users
        $result = $this->query('SELECT `email`, `passwd` FROM `' . $this->dbOldPrefix . 'LINK` WHERE `email` != "" AND `passwd` != ""', $this->dbOld);

        // Set-up new database
        $this->setNewDb();

        // Get list of arfooo administrators
        $result2 = $this->query('SELECT `email` FROM `' . $this->dbNewPrefix . 'users` WHERE `email` != "" AND `role` = "administrator"', $this->dbNew);

        // Loop through new users to build array as needed
        while ($row = mysql_fetch_assoc($result2))
        {
            $users[] = $row['email'];
        }

        // Delete non-admin users from arfooo and modify email field to varchar(100)
        $this->query('DELETE FROM `' . $this->dbNewPrefix . 'users` WHERE `role` != "administrator"', $this->dbNew);
        $this->query('ALTER TABLE `' . $this->dbNewPrefix . 'users` MODIFY `email` VARCHAR(100)', $this->dbNew);

        // Set table name and table prefix
        $this->qObiect->tableName = 'users';
        $this->qObiect->dbPrefix = $this->dbNewPrefix;

        // Loop through linker users to store them in arfooo
        while ($row = mysql_fetch_assoc($result))
        {
            // Old user's email
            $email = $row['email'];

            // If email already exists then no need to add it
            if (in_array($email, $users))
            {
                continue;
            }

            // Reset insert query's object
            $this->qObiect->reset();

            // Set email, role and password
            $this->qObiect->add('email', $email);
            $this->qObiect->add('role', 'webmaster');
            $this->qObiect->add('password', md5($row['passwd']));

            // Build SQL query
            $sql = $this->qObiect->getSQL();
            $sql = iconv('ISO-8859-1', 'utf-8//ignore', $sql);
            $sql = str_replace("\xc2\x80", "\xe2\x82\xac", $sql);

            // Run query
            $this->query($sql, $this->dbNew);

            // Store email in users array
            $users[] = $email;
        }

        // By default return true
        return true;
    }
}
